#include "history.h"

#define PATHLEN 200

typedef struct BattInfo_s{
	int present;
	int fullcap;        /* full capacity, only updated on restart */
	int charge;         /* charging (0), discharging (1), full (2), invalid (3) */
	int currcap;        /* current remaining capacity */
	int currrate;       /* current rate of discharge */
	char statepath[PATHLEN];    /* file name of state info */
	struct BattInfo_s * next;
} BattInfo;

typedef struct {
	int temperature;		/* temperature (-1 if no reading) */
	int critical_temp;  /* critical temperature */
	int passive_temp;   /* temperature at which to switch to passive */
	int speed;          /* processor speed, MHz */
	int cooling;        /* cooling CPU on/off */
	int state;			    /* thermal_zone_state */
	int batt;			      /* battery (0) or AC adapter (1) */
	int load;           /* system load average */
	BattInfo *batteries;
	int cum_charge;     /* cumulative charging state */
	int cum_remaining;  /* cumulative remaining capacity */
	int cum_rate;       /* cumulative discharge rate */
	int cum_fullcap;    /* cumulative full charge */
	int battery_percent; /* cumulative percent full */
	int remaining_hours; /* hours left for charge/discharge */
	int remaining_minutes; /* minutes left for charge/discharge */
	float current_rate; /* computed change in charge (signed) */
	char ac_path[PATHLEN];      /* path to sysfs ac adapter*/
	char thermal_path[PATHLEN];    /* path to sysfs thermal zone */
	History *charge_hist; /* History of battery charge */
} ACPInfo;

/* fill APMInfo with data */
void acquire_info(ACPInfo *acpinfo);
void compute_battStatus(ACPInfo *acpinfo);
void init_acpinfo(ACPInfo *acpinfo);
int power_init(void);
