#define _ISOC99_SOURCE
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <math.h>
#include "history.h"


#define RINGBUF_IND(hist, i) ((i+hist->length+hist->next_index)%hist->length)


/* helper: "high-resolution" timer with double value (should be good for
decades to come...).  Requires -lrt */
double double_time(void) {
	struct timespec ts;

	if (clock_gettime(CLOCK_MONOTONIC, &ts)) { /* trouble, return 0
		so timers fire when the clock is back */
		return 0.;
	}
	return ts.tv_sec+ts.tv_nsec*1e-9;
}


void history_clear(History *hist)
{
	int i;

	for (i=0; i<hist->length; i++) {
		hist->vals[i].val = HIST_NO_VAL;
	}
}

History *history_new(int history_length)
{
	History *hist=malloc(sizeof(History)+sizeof(Timedvalue)*history_length);

	if (hist) {
		hist->next_index = 0;
		hist->length = history_length;
		history_clear(hist);
	}
	return hist;
}

double history_average(History *hist)
{
	double sum=0;
	int num_vals=0, i;

	for (i=0; i<hist->length; i++) {
		if (hist->vals[i].val!=HIST_NO_VAL) {
			num_vals++;
			sum += hist->vals[i].val;
		}
	}
	if (num_vals) {
		return sum/num_vals;
	}
	return NAN;
}


double history_diffaverage(History *hist)
// returns an estimate for the discharge rate over the interval
// of observation, i.e., between the oldest and the newest entry.
{
	double oldest_time = 1e300, newest_time = 0;
	double oldest_val = 0, newest_val = 0;
	int i;

	for (i=0; i<hist->length-1; i++) {
		if (hist->vals[i].val==HIST_NO_VAL) {
			continue;
		}

		if (hist->vals[i].time<oldest_time) {
			oldest_time = hist->vals[i].time;
			oldest_val = hist->vals[i].val;
		}
		if (hist->vals[i].time>newest_time) {
			newest_time = hist->vals[i].time;
			newest_val = hist->vals[i].val;
		}
	}

	if (oldest_time<newest_time) {
		return (oldest_val-newest_val)/(oldest_time-newest_time);
	}
	return NAN;
}

int history_countitems(History *hist)
{
	int i, ct=0;

	for (i=0; i<hist->length; i++) {
		if (hist->vals[i].val!=HIST_NO_VAL) {
			ct++;
		}
	}
	return ct;
}

void history_addval(History *hist, double val)
{
	hist->vals[hist->next_index].val = val;
	hist->vals[hist->next_index].time = double_time();
	hist->next_index = (hist->next_index+1)%hist->length;
}


double history_curval(History *hist) {
	return hist->vals[RINGBUF_IND(hist, -1)].val;
}
